<?php
namespace HCM\Geral\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO;
use HCM\Util\HCMParameters;

class PerfilAcessoController extends Crud {

    protected $dataSourceName = "perfilacesso";
    protected $dataSourceNameGet = "perfilacesso_get";
    protected $perfilAcessoRepository;
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    
    public function __construct(Manager $dataSourceManager, $environment, ParameterBag $parameterBag, HCMParameters $hcmParameter, $perfilAcessoRepository) {
        $this->perfilAcessoRepository = $perfilAcessoRepository;
        $this->hcmParameters = $hcmParameter;
        $this->parameterBag = $parameterBag;
        parent::__construct($dataSourceManager, $environment, $parameterBag);
    }
    
    public function getPerfilAcesso($request, $response){
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        $index = 1;
        try {
            $maxNRPERFILACESSO = $this->perfilAcessoRepository->getMaxNrPerfilAcesso();
        }
        catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage()));
            $maxNRPERFILACESSO = 1;
        }

        foreach ($request->getDataSet()->getRows() as $row) {
            if ($row->get("__is_new")) {
                $row->set("NRPERFILACESSO", $maxNRPERFILACESSO+$index);
                $index++;
            }
            $firstRow = current($request->getDataSet()->getRows());
            $firstRow->set("NRUSERGROUP", null);
            $firstRow->set("NRPERFACESSOPAI", null);
        }

        parent::save($request, $response);
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        $perfisToDelete = array();
        foreach ($rows as $row) {
            $perfisToDelete[] = $row->get("NRPERFILACESSO");
        }
        $params = array (
            "NRORG" =>  $this->environment->getOrganizationId()
        );
        try {
            $this->perfilAcessoRepository->deleteAssocietedRules($perfisToDelete, $params);
            $this->perfilAcessoRepository->deleteAssocietedPerfAcessParc($perfisToDelete, $params);
        }
        catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage()));
        }

        parent::delete($request, $response);
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}