<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S2220Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function getItensParaEnvioS2220($params) {
        $sql = <<<SQL
SELECT FM.NRFICHAMEDICA as NRCHAVE,
       VH.NRVINCULOM,
       '' as NRTABELA,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM SST_FICHAMEDICA FM
  JOIN GPE_VINCULOM VM ON VM.NRORG = FM.NRORG
                      AND FM.NRVINCULOM = VM.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
  LEFT JOIN ESTRUTURAH EH ON EH.NRORG = VH.NRORG
                         AND EH.NRESTRUTURAM = VH.NRESTRUTLEGAL
                         AND EH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                   WHERE H1.NRORG = EH.NRORG
                                                     AND H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2220'
               /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
               JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                  AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                  AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                  AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                             WHERE H1.NRORG = EVH.NRORG
                                                               AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                               AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE
             UNION ALL
             SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_NAOPERIODICO EC
              WHERE EC.NRORG = :P_NRORG
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NMEVENTO = 'S-2220'
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VH.NRVINCULOM
 WHERE FM.NRORG = :P_NRORG
   AND SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
   AND (FM.NRFICHAMEDICA = :P_NRFICHAMEDICA OR :P_NRFICHAMEDICA IS NULL)
   AND (VH.DTMESCOMPETENC >= :P_DTREFNAOPERIOD OR :P_DTREFNAOPERIOD IS NULL)
 ORDER BY 1,5
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
            "P_NRFICHAMEDICA" => isset($params['NRFICHAMEDICA']) ? $params['NRFICHAMEDICA'] : null,
            "P_DTREFNAOPERIOD" => isset($params['DTREFNAOPERIOD']) ? $params['DTREFNAOPERIOD'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }

    public function getDadosS2220($parametros) {
        try {
            $dados = $this->getDadosPrincipais($parametros['NRENVIO'], $parametros['NRCHAVE'], $parametros['NRESTRUTURAM']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosPrincipais($nrenvio, $nrchave, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2220*/
SELECT FM.NRFICHAMEDICA CHAVE,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,    
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,  
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) AS matricula,
       DECODE(VM.NRTIPOVINCULOM,1,'',CE.CDESOCIAL) AS codCateg,
       --exMedOcup
       NE.CDESOCIAL AS tpExameOcup,
       --aso
       TO_CHAR(FM.DTHRPREVFICHAMED,'YYYY-MM-DD') AS dtAso,
       TRE.IDAPTO AS resAso,   
       --getDadosS2220exame
       --medico
       PHRM.NMPESSOA AS nmMed,
       PHRM.NRINSCRCONSREG AS nrCRM,
       PHRM.SGUFCONSREG AS ufCRM,
       --respMonit
       REPLACE(REPLACE(PHRM.NRCPFPESSOA,'.'),'-') cpfResp,
       PHRM.NMPESSOA AS nmResp,
       PHRM.NRINSCRCONSREG AS respMonit_nrCRM,
       PHRM.SGUFCONSREG AS respMonit_ufCRM
  FROM SST_FICHAMEDICA FM 
  /*EXAME*/
  LEFT JOIN SST_NATUREZAEXAME NE ON (NE.NRORG = FM.NRORG OR NE.NRORG = 0) AND NE.NRNATUREZAEXAME = FM.NRNATUREZAEXAME
  LEFT JOIN SST_TABRESULTEXAME TRE ON (TRE.NRORG = FM.NRORG OR TRE.NRORG = 0) AND TRE.NRTABRESULTEXAME = FM.NRTABRESULTEXAME
  /*Medico Responsavel*/
  LEFT JOIN GPE_PESSOA PRM ON PRM.NRPARCNEGOCIO = FM.NRPARCNEGOCIO
                          AND PRM.NRORG         = FM.NRORG
  LEFT JOIN GPE_PESSOAH PHRM ON PRM.NRPESSOA = PHRM.NRPESSOA
                            AND PRM.NRORG    = PHRM.NRORG
                            AND PHRM.NRINSCRCONSREG IS NOT NULL
                            AND PHRM.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) 
							                             FROM GPE_PESSOAH
                                                         WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                           AND GPE_PESSOAH.NRPESSOA = PHRM.NRPESSOA
                                                           AND GPE_PESSOAH.NRORG    = PHRM.NRORG)
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOM VM ON VM.NRORG = FM.NRORG
                      AND VM.NRVINCULOM = FM.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG      = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VH.NRORG)
  /*Dados do Empregado - Pessoa*/
  JOIN GPE_PESSOAH PH
    ON PH.NRPESSOA = VM.NRPESSOA
   AND PH.NRORG    = VM.NRORG
   AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                             WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_PESSOAH.NRPESSOA = PH.NRPESSOA
                               AND GPE_PESSOAH.NRORG = PH.NRORG)
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*Categoria Trabalhador*/
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
 WHERE FM.NRORG = :P_NRORG
   AND FM.NRFICHAMEDICA = :P_NRCHAVE
   and PRM.NRPARCNEGOCIO = FM.NRPARCNEGOCIO
   AND PRM.NRORG         = :P_NRORG
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getDadosS2220exame($parametros) {
        try {
            $dados = $this->getDadosExame($parametros['CHAVE']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosExame($nrfichamedica) {
        $sql = <<<SQL
/*getDadosS2220exame*/
SELECT TO_CHAR(EFM.DTREALIZACAOEXAME,'YYYY-MM-DD') AS dtExm,
       EFM.CDPROCTUSS AS procRealizado,
       EFM.OBSPROCTUSS AS obsProc,
       DECODE(EFM.IDEXAMEREFSEQ, 'R', 1, 'S', 2) AS ordExame,
       DECODE(TRE.IDRESULTEXAME, 'N', 1, 'A', 2,'E', 3, 'G', 4) indResult
  FROM SST_EXAMEFICHAMED EFM
  LEFT JOIN SST_TABRESULTEXAME TRE
    ON (TRE.NRORG = EFM.NRORG OR TRE.NRORG = 0)
   AND TRE.NRTABRESULTEXAME = EFM.NRTABRESULTEXAME
 WHERE EFM.NRORG = :P_NRORG
   AND EFM.NRFICHAMEDICA = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_NRCHAVE" => $nrfichamedica,
        );
        return $this->connection->fetchAll($sql, $params);
    }

}